Program FileInfo;

{$APPTYPE CONSOLE}
{$R 'FileInfo.res' 'FileInfo.rc'}

Uses
  SysUtils,
  Main In 'Main.pas',
  RegExpr In 'RegExpr.pas';

Var
  Separator, Mask, MaskReplace: String;
  Limit:                        Integer;
Begin
  {$IFDEF DEBUG}
  ReportMemoryLeaksOnShutDown := true; //si debug on affiche les problmes de libration de mmoire
  {$ENDIF}
  Error := 0;
  Try
    Try
      switchPrefixes := ['-', '/'];

      If ParamCount < 1 Then
      Begin
        Error := 2;
        ConsoleString('Need more parameter, /h for see help');
        Exit;
      End;

    //want to see help
      If FindCmdLineSwitch('?', switchPrefixes, True) Or
        FindCmdLineSwitch('h', switchPrefixes, True) Or
        FindCmdLineSwitch('help', switchPrefixes, True) Then
      Begin
        WriteHelp(GetParamVal('language'));
      {$IFDEF DEBUG}
      Readln;
      {$ELSE}
        Exit;
      {$ENDIF}
      End;

      //last param is text or file if not found file exit
      If (Not FindCmdLineSwitch('text', switchPrefixes, True) And Not FileExists(ParamStr(ParamCount))) And
        (FindCmdLineSwitch('del', switchPrefixes, True) And Not DirectoryExists(ParamStr(ParamCount))) Then
      Begin
        Error := 2;
        ConsoleString('last parameter incorrect: "' + ParamStr(ParamCount) + '"');
        Exit;
      End;

      //return version info
      If FindCmdLineSwitch('version', switchPrefixes, True) Then
      Begin
      //set default separator if none is set
        Separator := GetParamVal('separator');
        If Separator = '' Then
          Separator := '.';

        //if needly some number or full version number
        Try
          If FindCmdLineSwitch('n1', switchPrefixes, True) Or
            FindCmdLineSwitch('n2', switchPrefixes, True) Or
            FindCmdLineSwitch('n3', switchPrefixes, True) Or
            FindCmdLineSwitch('n4', switchPrefixes, True) Then
            ConsoleString(ApplicationVersion(ParamStr(ParamCount), Separator,
              FindCmdLineSwitch('n1', switchPrefixes, True),
              FindCmdLineSwitch('n2', switchPrefixes, True),
              FindCmdLineSwitch('n3', switchPrefixes, True),
              FindCmdLineSwitch('n4', switchPrefixes, True)))
          Else
            ConsoleString(ApplicationVersion(ParamStr(ParamCount), Separator));
          Exit;
        Except
          on E: Exception Do
          Begin
            Error := 2;
            ConsoleString('Exception class: ' + E.ClassName + ' with message: ' + E.Message);
          End;
        End;
      End;

      If FindCmdLineSwitch('haval', switchPrefixes, True) Then
      Begin
        ConsoleString(GetHaval(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('md4', switchPrefixes, True) Then
      Begin
        ConsoleString(GetMD4(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('md5', switchPrefixes, True) Then
      Begin
        ConsoleString(GetMD5(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('ripemd128', switchPrefixes, True) Then
      Begin
        ConsoleString(GetRipeMD128(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('ripemd160', switchPrefixes, True) Then
      Begin
        ConsoleString(GetRipeMD160(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('sha1', switchPrefixes, True) Then
      Begin
        ConsoleString(GetSHA1(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('sha256', switchPrefixes, True) Then
      Begin
        ConsoleString(GetSHA256(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('sha384', switchPrefixes, True) Then
      Begin
        ConsoleString(GetSHA384(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('sha512', switchPrefixes, True) Then
      Begin
        ConsoleString(GetSHA512(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('tiger', switchPrefixes, True) Then
      Begin
        ConsoleString(GetTiger(ParamStr(ParamCount)));
        Exit;
      End;

      If FindCmdLineSwitch('size', switchPrefixes, True) Then
      Begin
        If FindCmdLineSwitch('ko', switchPrefixes, True) Then
          ConsoleString(GetSize(ParamStr(ParamCount), rsKO, FindCmdLineSwitch('round', switchPrefixes, True)))
        Else
        If FindCmdLineSwitch('mo', switchPrefixes, True) Then
          ConsoleString(GetSize(ParamStr(ParamCount), rsMO, FindCmdLineSwitch('round', switchPrefixes, True)))
        Else
        If FindCmdLineSwitch('go', switchPrefixes, True) Then
          ConsoleString(GetSize(ParamStr(ParamCount), rsGO, FindCmdLineSwitch('round', switchPrefixes, True)))
        Else
          ConsoleString(GetSize(ParamStr(ParamCount), rsO, FindCmdLineSwitch('round', switchPrefixes, True)));
        Exit;
      End;

      If FindCmdLineSwitch('match', switchPrefixes, True) Then
      Begin
        Mask := GetParamVal('mask');
        If Mask = '' Then
        Begin
          Error := 2;
          ConsoleString('"Mask" parameter is not found cannot continue.');
          Exit;
        End;
        Limit := 3;
        If FindCmdLineSwitch('text', switchPrefixes, True) Then
          Inc(Limit);
        If FindCmdLineSwitch('full', switchPrefixes, True) Then
          Inc(Limit);
        If GetParamVal('separator') <> '' Then
          Inc(Limit);
        If ParamCount < Limit Then
        Begin
          Error := 2;
          ConsoleString('Parameter number to less is not found cannot continue.');
          Exit;
        End;
        ConsoleString(
          GetMatch(ParamStr(ParamCount), Mask, FindCmdLineSwitch('text', switchPrefixes, True),
          FindCmdLineSwitch('full', switchPrefixes, True), GetParamVal('separator'))
          );
        Exit;
      End;

      If FindCmdLineSwitch('replace', switchPrefixes, True) Then
      Begin
        Mask := GetParamVal('mask');
        If Mask = '' Then
        Begin
          Error := 2;
          ConsoleString('"Mask" parameter is not found cannot continue.');
          Exit;
        End;
        MaskReplace := GetParamVal('maskreplace');
        If MaskReplace = '' Then
        Begin
          Error := 2;
          ConsoleString('"MaskReplace" parameter is not found cannot continue.');
          Exit;
        End;
        Limit := 4;
        If FindCmdLineSwitch('text', switchPrefixes, True) Then
          Inc(Limit);
        If FindCmdLineSwitch('usesubstitution', switchPrefixes, True) Then
          Inc(Limit);
        If FindCmdLineSwitch('save', switchPrefixes, True) Then
          Inc(Limit);
        If ParamCount < Limit Then
        Begin
          Error := 2;
          ConsoleString('Parameter number to less cannot continue.');
          Exit;
        End;
        ConsoleString(
          ReplaceMatch(ParamStr(ParamCount), Mask, MaskReplace,
          FindCmdLineSwitch('text', switchPrefixes, True),
          FindCmdLineSwitch('usesubstitution', switchPrefixes, True),
          FindCmdLineSwitch('save', switchPrefixes, True),
          GetParamVal('linebreak'),
          GetParamVal('modifier'))
          );
        Exit;
      End;

      If FindCmdLineSwitch('del', switchPrefixes, True) Then
      Begin
        Try
          If Not DeleteFileToRecycleBin(ParamStr(ParamCount)) Then
          Begin
            Error := 2;
            ConsoleString('Cannot delete "' + ParamStr(ParamCount) + '"');
            Exit;
          End;
          Exit;
        Except
          on E: Exception Do
          Begin
            Error := 2;
            ConsoleString('Exception class: ' + E.ClassName + ' with message: ' + E.Message);
          End;
        End;
      End;

    //if not exit so there are unknow error we send code 1
      Error := 1;
    Except
      on E: Exception Do
      Begin
        Error := 1; //unknow error
        ConsoleString(E.ClassName + ': ' + E.Message);
        {$IFDEF DEBUG}
        Readln;
        {$ENDIF}
      End;
    End;
  Finally
    If Error > 0 Then
      Halt(Error);
    {$IFDEF DEBUG}
    Readln;
    {$ENDIF}
  End;
End.

